<?php 

/** 
* Plugin Name: WEB Push уведомления (firebase)
* Description: Отправка Push уведомлений пользователям
* Version: 1.0.0
* Author: alex-borisi
*/ 

// Запретим прямое обращение
if (!defined('ROOTPATH')) die('Core not loaded'); 

// URL к папке с плагином
define('FBPUSH_PLUGIN_URI', str_replace(ROOTPATH, ds_site_url(), dirname(__FILE__))); 
define('FBPUSH_PLUGIN_NAME', preg_replace('/.*\//', '', dirname(__FILE__))); 
define('FBPUSH_API_URL', 'https://fcm.googleapis.com/fcm/send'); 

add_event('ds_init', 'fbpush_init'); 
function fbpush_init() 
{
	$options = get_options('fbpush');

	if (isset($_GET['firebase_hash']) && isset($_GET['firebase_id']) && isset($_GET['firebase_action'])) {
		$firebase_hash = $_GET['firebase_hash']; 
		$firebase_id = (int) $_GET['firebase_id']; 
		$firebase_action = $_GET['firebase_action']; 

		if ($firebase_action == 'read') {
			$firebase = db::fetch("SELECT * FROM user_firebase WHERE id = " . $firebase_id); 

			if ($firebase_hash === hash('sha256', $firebase['token'])) {
				if (!is_user()) {
					$_SESSION['id_user'] = $firebase['user_id'];
				}

				db::update('user_firebase', array(
					'posted' => 0, 
					'browser' => $_SERVER['HTTP_USER_AGENT'], 
				), array(
					'id' => $firebase_id, 
				)); 
			}
		}
	}

	if (!is_user()) return ; 

	if (isset($_GET['firebase_subscribe'])) {
		$token = $_POST['token'];

		if (is_user()) {
			$user_id = get_user_id(); 
			$firebase = db::fetch("SELECT * FROM user_firebase WHERE user_id = '" . $user_id . "' AND `browser` = '" . my_esc($_SERVER['HTTP_USER_AGENT']) . "'"); 

			$data = array(
				'browser' => $_SERVER['HTTP_USER_AGENT'], 
				'token'   => $token, 
				'user_id' => $user_id, 
			); 

			if (isset($firebase['id'])) {
				db::update('user_firebase', array(
					'token' => $token, 
				), array(
					'id' => $firebase['id'], 
				)); 
			} else {
				db::insert('user_firebase', $data); 
			}

			die(json_encode(array('ok' => 1)));
		}
	}

	add_event('init_head_theme', 'fbpush_styles_load', 1); 
	add_event('init_foot_theme', 'fbpush_scripts_load', 11); 
	add_event('ds_admin_init', 'fbpush_settings'); 
	add_event('init_head', 'fbpush__header_app_id'); 
	add_event('init_foot_theme', 'fbpush_modal_subscribe'); 
	add_event('mail_send', 'fbpush_notification_send'); 
	
	ds_rewrite_rule('firebase-messaging-sw\.js', 'fbpush_firebase_messaging_sw'); 
}

function fbpush_notification_send($from, $to, $msg) 
{
	$msg = output_text($msg); 
	fb_push($to, __('Сообщение от %s', get_user_nick($from)), $msg, ds_site_url() . '/mail.php?id=' . $from, get_avatar_uri($from, 128)); 
}

function fbpush_modal_subscribe() 
{
	echo '<div class="fbpush">'; 
	echo '<div class="fbpush-title">Подписка на уведомления</div>'; 
	echo '<div class="fbpush-text">Получайте Push уведомления о новых сообщениях!</div>'; 
	echo '<a class="fbtn" id="fbpush-not">Нет, спасибо</a>'; 
	echo '<a class="fbtn fbtn-subscribe" id="fbpush-subscribe">Подписаться</a>'; 
	echo '</div>'; 
}

/**
* Отправляет сообщение в Firebase
* @return (int) count send messages
*/ 
function fb_push($to_id, $title, $content, $link, $icon) 
{
	$options = get_options('fbpush');
	$q = db::query("SELECT * FROM user_firebase WHERE user_id = " . $to_id); 

	$count = 0; 

	while($notif = $q->fetch_assoc()) : 
		$data = array(
    		'to' => $notif['token'], 
    		'notification' => array(
    			'title' => $title, 
    			'body' => html_entity_decode(strip_tags($content)), 
    			'icon' => $icon, 
    			'click_action' => $link . '&firebase_action=read&firebase_id=' . $notif['id'] . '&firebase_hash=' . hash('sha256', $notif['token']),
    		), 
    	);

        $ch = curl_init();
        $array = array(
        	CURLOPT_URL => FBPUSH_API_URL, 
        	CURLOPT_HTTPHEADER => array(
        		'Content-Type: application/json', 
        		'Authorization: key=' . preg_replace('/\s+/', '', $options['fbpush_api_key']) 
        	), 
        	CURLOPT_POSTFIELDS => json_encode($data), 
        	CURLOPT_RETURNTRANSFER => TRUE, 
        	CURLOPT_FOLLOWLOCATION => TRUE,
        );
        curl_setopt_array($ch, $array);
        curl_exec($ch);
        curl_close($ch);
        $count++; 
	endwhile;

	return $count; 
}

function fbpush_firebase_messaging_sw() 
{
	$options = get_options('fbpush');

	header('Content-Type:text/javascript;charset=utf-8');
	header("Cache-control: public");
	header("Expires: " . gmdate("D, d M Y H:i:s", time() + 60 * 60 * 24 * 365) . " GMT");

	echo "importScripts('https://www.gstatic.com/firebasejs/3.6.8/firebase-app.js');\r\n";
	echo "importScripts('https://www.gstatic.com/firebasejs/3.6.8/firebase-messaging.js');\r\n";
	echo "firebase.initializeApp({\r\n";
	echo "    messagingSenderId: '" . $options['fbpush_api_id'] . "'\r\n"; 
	echo "});\r\n";
	echo "const messaging = firebase.messaging();";
	exit;
}

function fbpush__header_app_id() 
{
	$options = get_options('fbpush');
	echo '<script>var firebase_sender_ID = \'' . $options['fbpush_api_id'] . '\';</script>';
}

function fbpush_scripts_load() 
{
	ds_theme_script_add('//www.gstatic.com/firebasejs/3.6.8/firebase.js', 'firebase', '3.6.8'); 
	ds_theme_script_add(FBPUSH_PLUGIN_URI . '/js/firebase_push.js', 'firebase_push', '1.0.1'); 
}

function fbpush_styles_load() 
{
	ds_theme_style_add(FBPUSH_PLUGIN_URI . '/css/style.css', 'fbpush-main', '1.0.1', 'all');
}

function fbpush_settings() 
{
    add_settings_page('fbpush_settings', 'Настройки Push уведомлений (firebase)', 'adm_set_sys', '', 'Push уведомления', true); 
    add_event('ds_admin_settings_fbpush_settings_init', 'fbpush_settings_fields'); 
}

function fbpush_settings_fields() 
{
    add_settings_section('fbpush_settings', 'fbpush_settings_section', array(
        'option_type' => 'fbpush', 
    )); 

    $fields = array(
    	array(
	        'title' => __('Ключь сервера (App Key)'), 
	        'name' => 'fbpush_api_key', 
	        'type' => 'textarea', 
    	), 
    	array(
	        'title' => __('Идентификатор отправителя (App ID)'), 
	        'name' => 'fbpush_api_id', 
    	), 
    ); 

    foreach($fields AS $field) {
	    add_settings_field('fbpush_settings', 'fbpush_settings_section', array_merge(array(
	        'type' => 'text', 
	        'value' => '%value', 
	        'checked' => '%value',  
	    ), $field));    	
    }
}

/**
* Создание таблиц в БД для плагина
*/ 
add_event('ds_plugin_' . FBPUSH_PLUGIN_NAME . '_pre_activation', 'fbpush_activation'); 
function fbpush_activation() 
{
	$result = db::is_exsits_column('user_firebase', 'id'); 
	
	if ($result === false) {
		db::query("CREATE TABLE IF NOT EXISTS `user_firebase` (
				  `user_id` int(11) NOT NULL,
				  `token` varchar(512) NOT NULL,
				  `id` int(11) NOT NULL,
				  `last_send` int(11) NOT NULL,
				  `browser` varchar(255) DEFAULT NULL,
				  `posted` int(11) NOT NULL DEFAULT '0'
				) ENGINE=InnoDB DEFAULT CHARSET=utf8;");

		db::query("ALTER TABLE `user_firebase` ADD UNIQUE KEY `id` (`id`)");
		db::query("ALTER TABLE `user_firebase` MODIFY `id` int(11) NOT NULL AUTO_INCREMENT");
	}
}