
firebase.initializeApp({
    messagingSenderId: window.firebase_sender_ID
});

function getCookieFBPush(name) {
  var matches = document.cookie.match(new RegExp(
    "(?:^|; )" + name.replace(/([\.$?*|{}\(\)\[\]\\\/\+^])/g, '\\$1') + "=([^;]*)"
  ));
  return matches ? decodeURIComponent(matches[1]) : undefined;
}

function setCookieFBPush(name, value, options) {
  options = options || {};

  var expires = options.expires;

  if (typeof expires == "number" && expires) {
    var d = new Date();
    d.setTime(d.getTime() + expires * 1000);
    expires = options.expires = d;
  }
  if (expires && expires.toUTCString) {
    options.expires = expires.toUTCString();
  }

  value = encodeURIComponent(value);

  var updatedCookie = name + "=" + value;

  for (var propName in options) {
    updatedCookie += "; " + propName;
    var propValue = options[propName];
    if (propValue !== true) {
      updatedCookie += "=" + propValue;
    }
  }

  document.cookie = updatedCookie;
}

// браузер поддерживает уведомления
// вообще, эту проверку должна делать библиотека Firebase, но она этого не делает
if ('Notification' in window) {
    var messaging = firebase.messaging();
    var fbpush = getCookieFBPush('fbpush'); 

    if (Notification.permission === 'default' && fbpush != 'not') {
        setTimeout(function() {
            $('.fbpush').addClass('fbpush-view');
        }, 500); 
    }
    
    console.log('Permission: ' + Notification.permission); 

    // пользователь уже разрешил получение уведомлений
    // подписываем на уведомления если ещё не подписали
    if (Notification.permission === 'granted') {
        fb_subscribe();
    }

    // по клику, запрашиваем у пользователя разрешение на уведомления
    // и подписываем его
    $('#fbpush-subscribe').on('click', function() {
        fb_subscribe();
    });

    $('#fbpush-not').on('click', function() {
        setCookieFBPush('fbpush', 'not', {
            expires: 60 * 60 * 24
        }); 
        $('.fbpush').fadeOut();
    });
}

function fb_subscribe() {
    $('.fbpush').addClass('fbpush-loading');
    $('.fbpush-text').text('Нажмите "Разрешить" чтобы завершить подписку!'); 

    // запрашиваем разрешение на получение уведомлений
    messaging.requestPermission()
        .then(function () {
            // получаем ID устройства
            messaging.getToken()
                .then(function (currentToken) {
                    console.log(currentToken);

                    if (currentToken) {
                        sendTokenToServer(currentToken);
                    } else {
                        console.warn('Не удалось получить токен.');
                        setTokenSentToServer(false);
                    }
                })
                .catch(function (err) {
                    console.warn('При получении токена произошла ошибка.', err);
                    setTokenSentToServer(false);
                });
    })
    .catch(function (err) {
        if (getCookieFBPush('fbpush') != 'not') {
            setCookieFBPush('fbpush', 'yes', {
                expires: -1, 
            });             
        }
        console.warn('Не удалось получить разрешение на показ уведомлений.', err);
    });
}

// отправка ID на сервер
function sendTokenToServer(currentToken) {
    if (!isTokenSentToServer(currentToken)) {
        console.log('Отправка токена на сервер...');

        var url = '?firebase_subscribe'; // адрес скрипта на сервере который сохраняет ID устройства
        $.post(url, {
            token: currentToken
        });

        setTimeout(function() {
            $('.fbpush-text').text('Вы успешно подписались! Теперь Вы сможете получать уведомления о новых сообщениях.'); 
            $('.fbpush').addClass('fbpush-success').removeClass('fbpush-loading'); 

            setTimeout(function() {
                $('.fbpush').fadeOut(); 
            }, 2500); 
        }, 1000); 
        

        setCookieFBPush('fbpush', 'yes', {
            expires: 60 * 60 * 24 * 365 * 12
        }); 
        setTokenSentToServer(currentToken);
    } else {
        console.log('Токен уже отправлен на сервер.');
    }
}

// используем localStorage для отметки того,
// что пользователь уже подписался на уведомления
function isTokenSentToServer(currentToken) {
    return window.localStorage.getItem('sentFirebaseMessagingToken') == currentToken;
}

function setTokenSentToServer(currentToken) {
    window.localStorage.setItem(
        'sentFirebaseMessagingToken',
        currentToken ? currentToken : ''
    );
}